@@ -5,7 +5,7 @@ package App::Greple::md;
 use 5.024;
 use warnings;
 
-our $VERSION = "0.9901";
+our $VERSION = "0.9902";
 
 =encoding utf-8
 
@@ -83,7 +83,7 @@ C<E<0x2524>>, C<E<0x253C>>).
 
     greple -Mmd --no-rule -- file.md
 
-=head2 B<--cm> I<LABEL>=I<SPEC>
+=head2 B<--colormap> I<LABEL>=I<SPEC>, B<--cm> I<LABEL>=I<SPEC>
 
 Override the color for a specific element.  I<LABEL> is one of
 the color labels listed in L</COLOR LABELS>.  I<SPEC> follows
@@ -93,6 +93,25 @@ function specs via L<Getopt::EX::Colormap>.
     greple -Mmd --cm h1=RD -- file.md
     greple -Mmd --cm bold='${base}D' -- file.md
 
+=head2 B<--heading-markup>[=I<STEPS>], B<--hm>[=I<STEPS>]
+
+Control inline markup processing inside headings.  By default,
+headings are rendered with uniform heading color without processing
+bold, italic, strikethrough, or inline code inside them.  Links
+are always processed as OSC 8 hyperlinks regardless of this option.
+
+Without an argument, all inline formatting becomes visible within
+headings using cumulative coloring.  With an argument, only the
+specified steps are processed inside headings.  Steps are separated
+by colons.
+
+Available steps: C<inline_code>, C<horizontal_rules>, C<bold>,
+C<italic>, C<strike>.
+
+    greple -Mmd --hm -- file.md                  # all markup
+    greple -Mmd --hm=bold -- file.md              # bold only
+    greple -Mmd --hm=bold:italic -- file.md       # bold and italic
+
 =head2 B<--hashed> I<LEVEL>=I<VALUE>
 
 Append closing hashes to headings.  For example, C<### Title>
@@ -125,12 +144,14 @@ Nested hash parameters use dot notation:
 
 Available parameters:
 
-    mode          light or dark (default: light)
-    base_color    base color override
-    table         table formatting (default: 1)
-    rule          box-drawing characters (default: 1)
-    osc8          OSC 8 hyperlinks (default: 1)
-    hashed.h1-h6  closing hashes per level (default: 0)
+    mode            light or dark (default: light)
+    base_color      base color override
+    table           table formatting (default: 1)
+    rule            box-drawing characters (default: 1)
+    osc8            OSC 8 hyperlinks (default: 1)
+    heading_markup  inline markup in headings (default: 0)
+                    0=off, 1/all=all, or colon-separated steps
+    hashed.h1-h6    closing hashes per level (default: 0)
 
 =head2 OSC 8 Hyperlinks
 
@@ -143,37 +164,48 @@ Disable with:
 =head1 COLOR LABELS
 
 The following labels identify colorizable elements.  Use them
-with C<--cm> to customize colors or C<--show> to control
-visibility.
-
-=head2 Code
-
-    code_mark        Code delimiters (fences and backticks)
-    code_info        Fenced code block info string (language name)
-    code_block       Fenced code block body
-    code_inline      Inline code body
+with C<--colormap> (C<--cm>) to customize colors or C<--show> to control
+visibility.  Default values are shown as C<light / dark>.
+Colors follow L<Term::ANSIColor::Concise> format.
 
 =head2 Headings
 
-    h1 - h6          Heading levels 1 through 6
+    LABEL   LIGHT                    DARK
+    h1      L25D/${base};E           L00D/${base};E
+    h2      L25D/${base}+y20;E       L00D/${base}-y15;E
+    h3      L25DN/${base}+y30        L00DN/${base}-y25
+    h4      ${base}UD                ${base}UD
+    h5      ${base}+y20;U            ${base}-y20;U
+    h6      ${base}+y20              ${base}-y20
 
 =head2 Inline Formatting
 
-    bold             Bold (**text** or __text__)
-    italic           Italic (*text* or _text_)
-    strike           Strikethrough (~~text~~)
+    LABEL   LIGHT / DARK
+    bold    D
+    italic  I
+    strike  X
+
+=head2 Code
+
+    LABEL        LIGHT              DARK
+    code_mark    L20                L10
+    code_info    ${base_name}=y70   ${base_name}=y20
+    code_block   /L23;E             /L05;E
+    code_inline  L00/L23            L25/L05
 
 =head2 Block Elements
 
-    blockquote       Blockquote marker (>)
-    horizontal_rule  Horizontal rules (---, ***, ___)
-    comment          HTML comments (<!-- ... -->)
+    LABEL            LIGHT / DARK
+    blockquote       ${base}D
+    horizontal_rule  L15
+    comment          ${base}+r60
 
 =head2 Links
 
-    link             Inline links [text](url)
-    image            Images ![alt](url)
-    image_link       Image links [![alt](img)](url)
+    LABEL        LIGHT / DARK
+    link         I
+    image        I
+    image_link   I
 
 =head1 SEE ALSO
 
@@ -222,6 +254,7 @@ my $config = Getopt::EX::Config->new(
     base_color => '',  # override base color
     table      => 1,   # table formatting
     rule       => 1,   # box-drawing characters for tables
+    heading_markup => 0,  # inline formatting in headings
     hashed     => { h1 => 0, h2 => 0, h3 => 0, h4 => 0, h5 => 0, h6 => 0 },
 );
 
@@ -295,9 +328,15 @@ sub finalize {
     my($mod, $argv) = @_;
     $config->deal_with($argv,
                        "mode|m=s", "base_color|B=s", "table!", "rule!",
+                       "heading_markup|hm:s",
                        "hashed=s%",
-                       "cm=s" => \@opt_cm,
+                       "colormap|cm=s" => \@opt_cm,
                        "show=s%" => \%show);
+    # --hm with no argument gives "": treat as "all"
+    my $hm = $config->{heading_markup};
+    if (defined $hm && $hm eq '') {
+        $config->{heading_markup} = 'all';
+    }
 }
 
 sub setup_colors {
@@ -380,7 +419,7 @@ sub protect {
 
 sub restore {
     my $s = shift;
-    $s =~ s{$PR}{$protected[$1] // die "restore failed: index $1"}ge;
+    1 while $s =~ s{$PR}{$protected[$1] // die "restore failed: index $1"}ge;
     $s;
 }
 
@@ -404,111 +443,56 @@ my $LT = qr/(?:`[^`\n]*+`|\\.|[^`\\\n\]]++)+/;
 #
 # colorize() - the main function
 #
-# Receives entire file content in $_ (--begin with -G --all --need=0).
+# Receives entire file content in $_ (--begin with -G --filter).
 # Processes all patterns with multiline regexes.
 #
 
-sub colorize {
-    setup_colors();
-    @protected = ();
+#
+# Pipeline steps as code refs
+#
 
-    ############################################################
-    # 1. Fenced code blocks (multiline)
-    ############################################################
-
-    s{^( {0,3})(`{3,}|~{3,})(.*)\n((?s:.*?))^( {0,3})\2(\h*)$}{
-        my($oi, $fence, $lang, $body, $ci, $trail) = ($1, $2, $3, $4, $5, $6);
-        my $result = md_color('code_mark', "$oi$fence");
-        $result .= md_color('code_info', $lang) if length($lang);
-        $result .= "\n";
-        if (length($body)) {
-            $result .= join '', map { md_color('code_block', $_) }
-                split /(?<=\n)/, $body;
-        }
-        $result .= md_color('code_mark', "$ci$fence") . $trail;
-        protect($result)
-    }mge;
-
-    ############################################################
-    # 2. Inline code protection
-    ############################################################
-
-    s/(?<bt>`++)(((?!\g{bt}).)+)(\g{bt})/
-        protect(md_color('code_mark', $+{bt}) . md_color('code_inline', $2) . md_color('code_mark', $4))
-    /ge;
-
-    ############################################################
-    # 3. HTML comment protection (multiline)
-    ############################################################
-
-    s/(^<!--(?![->])(?s:.*?)-->)/protect(md_color('comment', $1))/mge;
-
-    ############################################################
-    # 4. Image links: [![alt](img)](url)
-    ############################################################
-
-    s{\[!\[($LT)\]\(([^)\n]+)\)\]\(<?([^>)\s\n]+)>?\)}{
-        protect(
-            osc8($2, md_color('image_link', "!"))
-            . osc8($3, md_color('image_link', "[$1]"))
-        )
-    }ge;
-
-    ############################################################
-    # 5. Images: ![alt](url)
-    ############################################################
-
-    s{!\[($LT)\]\(<?([^>)\s\n]+)>?\)}{
-        protect(osc8($2, md_color('image', "![$1]")))
-    }ge;
-
-    ############################################################
-    # 6. Links: [text](url) (not preceded by !)
-    ############################################################
-
-    s{(?<![!\e])\[($LT)\]\(<?([^>)\s\n]+)>?\)}{
-        protect(osc8($2, md_color('link', "[$1]")))
-    }ge;
-
-    ############################################################
-    # 7. Horizontal rule (before emphasis to prevent conflict)
-    ############################################################
-
-    if (active('horizontal_rule')) {
-        s/^([ ]{0,3}(?:[-*_][ ]*){3,})$/protect(md_color('horizontal_rule', $1))/mge;
-    }
-
-    ############################################################
-    # 8. Bold: **text** and __text__
-    ############################################################
-
-    if (active('bold')) {
-        s/(?<![\\`])\*\*.*?(?<!\\)\*\*/md_color('bold', $&)/ge;
-        s/(?<![\\`\w])__.*?(?<!\\)__(?!\w)/md_color('bold', $&)/ge;
-    }
-
-    ############################################################
-    # 9. Italic: _text_ and *text*
-    ############################################################
-
-    if (active('italic')) {
-        s/(?<![\\`\w])_(?:(?!_).)+(?<!\\)_(?!\w)/md_color('italic', $&)/ge;
-        s/(?<![\\`\*])\*(?:(?!\*).)+(?<!\\)\*(?!\*)/md_color('italic', $&)/ge;
-    }
-
-    ############################################################
-    # 10. Strikethrough: ~~text~~
-    ############################################################
-
-    if (active('strike')) {
-        s/(?<![\\`])~~.+?(?<!\\)~~/md_color('strike', $&)/ge;
-    }
-
-    ############################################################
-    # 11. Headings h6 -> h1 (cumulative over emphasis)
-    ############################################################
-
-    if (active('header')) {
+my %colorize = (
+    code_blocks => sub {
+        s{^( {0,3})(`{3,}|~{3,})(.*)\n((?s:.*?))^( {0,3})\2(\h*)$}{
+            my($oi, $fence, $lang, $body, $ci, $trail) = ($1, $2, $3, $4, $5, $6);
+            my $result = md_color('code_mark', "$oi$fence");
+            $result .= md_color('code_info', $lang) if length($lang);
+            $result .= "\n";
+            if (length($body)) {
+                $result .= join '', map { md_color('code_block', $_) }
+                    split /(?<=\n)/, $body;
+            }
+            $result .= md_color('code_mark', "$ci$fence") . $trail;
+            protect($result)
+        }mge;
+    },
+    comments => sub {
+        s/(^<!--(?![->])(?s:.*?)-->)/protect(md_color('comment', $1))/mge;
+    },
+    image_links => sub {
+        s{\[!\[($LT)\]\(([^)\n]+)\)\]\(<?([^>)\s\n]+)>?\)}{
+            protect(
+                osc8($2, md_color('image_link', "!"))
+                . osc8($3, md_color('image_link', "[$1]"))
+            )
+        }ge;
+    },
+    images => sub {
+        s{!\[($LT)\]\(<?([^>)\s\n]+)>?\)}{
+            protect(osc8($2, md_color('image', "![$1]")))
+        }ge;
+    },
+    links => sub {
+        s{(?<![!\e])\[($LT)\]\(<?([^>)\s\n]+)>?\)}{
+            protect(osc8($2, md_color('link', "[$1]")))
+        }ge;
+    },
+    inline_code => sub {
+        s/(?<bt>`++)(((?!\g{bt}).)+)(\g{bt})/
+            protect(md_color('code_mark', $+{bt}) . md_color('code_inline', $2) . md_color('code_mark', $4))
+        /ge;
+    },
+    headings => sub {
         my $hashed = $config->{hashed};
         for my $n (reverse 1..6) {
             next unless active("h$n");
@@ -517,25 +501,87 @@ sub colorize {
                 my $line = $1;
                 $line .= " $hdr"
                     if $hashed->{"h$n"} && $line !~ /\#$/;
-                md_color("h$n", restore($line));
+                protect(md_color("h$n", restore($line)));
             }mge;
         }
-    }
-
-    ############################################################
-    # 12. Blockquote: color only the > marker
-    ############################################################
-
-    if (active('blockquote')) {
+    },
+    horizontal_rules => sub {
+        s/^([ ]{0,3}(?:[-*_][ ]*){3,})$/protect(md_color('horizontal_rule', $1))/mge;
+    },
+    bold => sub {
+        s/(?<![\\`])\*\*.*?(?<!\\)\*\*/md_color('bold', $&)/ge;
+        s/(?<![\\`\w])__.*?(?<!\\)__(?!\w)/md_color('bold', $&)/ge;
+    },
+    italic => sub {
+        s/(?<![\\`\w])_(?:(?!_).)+(?<!\\)_(?!\w)/md_color('italic', $&)/ge;
+        s/(?<![\\`\*])\*(?:(?!\*).)+(?<!\\)\*(?!\*)/md_color('italic', $&)/ge;
+    },
+    strike => sub {
+        s/(?<![\\`])~~.+?(?<!\\)~~/md_color('strike', $&)/ge;
+    },
+    blockquotes => sub {
         s/^(>+\h?)(.*)$/md_color('blockquote', $1) . $2/mge;
+    },
+);
+
+#
+# Pipeline configuration
+#
+
+# Always before headings (protection + links)
+my @protect_steps = qw(code_blocks comments image_links images links);
+
+# Inline steps controlled by heading_markup
+my @inline_steps  = qw(inline_code horizontal_rules bold italic strike);
+
+# Always last
+my @final_steps   = qw(blockquotes);
+
+# Step-to-label mapping for active() check (unmapped = always active)
+my %step_label = (
+    headings         => 'header',
+    horizontal_rules => 'horizontal_rule',
+    bold             => 'bold',
+    italic           => 'italic',
+    strike           => 'strike',
+    blockquotes      => 'blockquote',
+);
+
+sub build_pipeline {
+    my $hm = $config->{heading_markup};
+
+    # heading_markup disabled: headings before all inline steps
+    if (!$hm) {
+        return (@protect_steps, 'headings', @inline_steps, @final_steps);
     }
 
-    ############################################################
-    # 13. Restore protected regions
-    ############################################################
+    # "all" or "1": all inline steps before headings
+    my %before;
+    if ($hm eq '1' || $hm =~ /^all$/i) {
+        %before = map { $_ => 1 } @inline_steps;
+    } else {
+        # "bold:italic" → collect word tokens, filter to valid inline steps
+        my %valid = map { $_ => 1 } @inline_steps;
+        %before = map { $_ => 1 } grep { $valid{$_} } ($hm =~ /(\w+)/g);
+    }
+
+    my @before_h = grep {  $before{$_} } @inline_steps;
+    my @after_h  = grep { !$before{$_} } @inline_steps;
+
+    return (@protect_steps, @before_h, 'headings', @after_h, @final_steps);
+}
+
+sub colorize {
+    setup_colors();
+    @protected = ();
+
+    for my $step (build_pipeline()) {
+        my $label = $step_label{$step};
+        next if $label && !active($label);
+        $colorize{$step}->();
+    }
 
     $_ = restore($_);
-
     $_;
 }
 
@@ -587,7 +633,5 @@ sub fix_separator {
 __DATA__
 
 option default \
-    -G --all --need=0 --filestyle=once --color=always \
-    --exit=0 \
-    -E '(*FAIL)' \
+    -G --filter --filestyle=once --color=always \
     --begin &__PACKAGE__::begin
